@@ -8,6 +8,7 @@
 #' @param clusterDiameter diameter in meters used to determine the number of clusters (see \code{addClustering})
 #' @param baseUrl base URL for data queries
 #' @param saveFile optional filename where raw CSV will be written
+#' @param flagAndKeep flag, rather then remove, bad data during the QC process
 #' @return Raw dataframe of WRCC data.
 #' @description Obtains monitor data from an AIRSIS webservice and converts
 #' it into a quality controlled, metadata enhanced "raw" dataframe
@@ -42,7 +43,8 @@ airsis_createRawDataframe <- function(startdate=20020101,
                                       provider=NULL, unitID=NULL,
                                       clusterDiameter=1000,
                                       baseUrl="http://xxxx.airsis.com/vision/common/CSVExport.aspx?",
-                                      saveFile=NULL) {
+                                      saveFile=NULL,
+                                      flagAndKeep=FALSE) {
   
   # Sanity checks
   if ( is.null(provider) ) {
@@ -71,16 +73,40 @@ airsis_createRawDataframe <- function(startdate=20020101,
   }
   
   # Read csv raw data into a dataframe
-  logger.info("Parsing data ...")
+  logger.debug("Parsing data ...")
   df <- airsis_parseData(fileString)
   
+  # Add source of raw data
+  if ( nrow(df) > 0 ) {
+    df$rawSource <- "AIRSIS"
+  }
+  
   # Apply monitor-appropriate QC to the dataframe
-  logger.info("Applying QC logic ...")
-  df <- airsis_qualityControl(df)
+  logger.debug("Applying QC logic ...")
+  df <- airsis_qualityControl(df, flagAndKeep=flagAndKeep)
   
   # Add clustering information to identify unique deployments
-  logger.info("Clustering ...")
-  df <- addClustering(df, lonVar='Longitude', latVar='Latitude', clusterDiameter=1000)
+  logger.debug("Clustering ...")
+  df <- addClustering(df, lonVar='Longitude', latVar='Latitude', clusterDiameter=clusterDiameter, flagAndKeep=flagAndKeep)
+  
+  # Rearrange columns to put QCFlag_* parameters at end if they exist
+  if ( flagAndKeep ) {
+    QC_columnNames <- c("QCFlag_anyBad",
+                        "QCFlag_reasonCode",
+                        "QCFlag_badLon",
+                        "QCFlag_badLat",
+                        "QCFlag_badType",
+                        "QCFlag_badFlow",
+                        "QCFlag_badAT",
+                        "QCFlag_badRHi",
+                        "QCFlag_badConcHr",
+                        "QCFlag_badDateAndTime",
+                        "QCFlag_duplicateHr")
+    # TODO: add intersection check here to remove those that do not exist in data
+    df_QC <- df[,QC_columnNames]
+    df_nonQC <- df[,-(which(names(df) %in% QC_columnNames))]
+    df <- cbind(df_nonQC,df_QC)
+  }
   
   return(df)
   
