@@ -37,7 +37,7 @@ epa_createDataDataframe <- function(df) {
   # combine the two dataframes together by doing a left join
   data <- dplyr::left_join(hourlyDF, pm25DF, by="datetime")
 
-  logger.debug("'data' dataframe has %d rows and %d columns", nrow(data), ncol(data))
+  logger.info("'data' dataframe has %d rows and %d columns", nrow(data), ncol(data))
   
   return(data)
 }
