@@ -1,9 +1,9 @@
 #' @keywords ws_monitor
 #' @export
 #' @title Calculate Distances From ws_monitor Monitors to Location of Interest
-#' @param ws_monitor emph{ws_monitor} object
-#' @param lon longitude of the location of interest
-#' @param lat latitude of the location of interest
+#' @param ws_monitor \emph{ws_monitor} object
+#' @param longitude longitude of the location of interest
+#' @param latitude latitude of the location of interest
 #' @description This function returns the distances (km) between monitoring sites and
 #' a location of interest. These distances can be used to create a mask identifying
 #' monitors within a certain radius of the location of interest.
@@ -19,8 +19,8 @@
 #' distance[closestIndex]
 #' N_M$meta[closestIndex,]
 
-monitor_distance <- function(ws_monitor, lon, lat) { 
-  distance <- distance(lon, lat, ws_monitor$meta$longitude, ws_monitor$meta$latitude)  
+monitor_distance <- function(ws_monitor, longitude, latitude) { 
+  distance <- distance(longitude, latitude, ws_monitor$meta$longitude, ws_monitor$meta$latitude)  
   names(distance) <- ws_monitor$meta$monitorID
   return(distance)
 }
