@@ -1,9 +1,9 @@
 #' @keywords ws_monitor
 #' @export
 #' @title Calculate Time Averages
-#' @param ws_monitor emph{ws_monitor} object
+#' @param ws_monitor \emph{ws_monitor} object
 #' @param ... additional arguments to be passed to \code{openair::timeAverage()}
-#' @return A emph{ws_monitor} object with data that have been proccessed by \code{openair::timeAverage()}.
+#' @return A \emph{ws_monitor} object with data that have been proccessed by \code{openair::timeAverage()}.
 #' @description This function extracts the \code{data} dataframe from \code{ws_monitor} object
 #' and renames the \code{'datetime'} column so that it can be processed by the \pkg{openair} package's
 #' \code{timeAverage()} function. (See that function for details.)
@@ -30,8 +30,8 @@ monitor_timeAverage <- function(ws_monitor, ...) {
   # Rebuild ws_monitor object
   names(averagedData)[1] <- 'datetime'
   ws_monitor <- list(meta=meta, data=averagedData)
-  ws_monitor <- structure(ws_monitor, class = c("ws_monitor", "list"))
   
-  return(ws_monitor)
+  return( structure(ws_monitor, class = c("ws_monitor", "list")) )
+  
 }
 
